searchNodes=[{"ref":"Phoenix.LiveView.html","title":"Phoenix.LiveView","type":"behaviour","doc":"LiveView provides rich, real-time user experiences with server-rendered HTML. LiveView programming model is declarative: instead of saying &quot;once event X happens, change Y on the page&quot;, events in LiveView are regular messages which may cause changes to its state. Once the state changes, LiveView will re-render the relevant parts of its HTML template and push it to the browser, which updates itself in the most efficient manner. This means developers write LiveView templates as any other server-rendered HTML and LiveView does the hard work of tracking changes and sending the relevant diffs to the browser. At the end of the day, a LiveView is nothing more than a process, that receives events as messages and updates its state. The state itself is nothing more than functional and immutable Elixir data structures. The events are either internal application messages (usually emitted by Phoenix.PubSub) or sent by the client/browser. LiveView provides many features that make it excellent to build rich, real-time user experiences: By building on top of Elixir processes and Phoenix.Channels, LiveView scales well vertically (from small to large instances) and horizontally (by adding more instances); LiveView is first rendered statically as part of regular HTTP requests, which provides quick times for &quot;First Meaningful Paint&quot; and also help search and indexing engines; LiveView performs diff tracking. If the LiveView state changes, it won&#39;t re-render the whole template, but only the parts affected by the changed state. This reduces latency and the amount of data sent over the wire; LiveView tracks static and dynamic contents. Any server-rendered HTML is made of static parts (i.e. that never change) and dynamic ones. On the first render, LiveView sends the static contents and in future updates only the modified dynamic contents are resent; (Coming soon) LiveView uses the Erlang Term Format to send messages to the client. This binary-based format is quite efficient on the server and uses less data over the wire; (Coming soon) LiveView includes a latency simulator, which allows you to simulate how your application behaves on increased latency and guides you to provide meaningful feedback to users while they wait for events to be processed; Furthermore, by keeping a persistent connection between client and server, LiveView applications can react faster to user events as there is less work to be done and less data to be sent compared to stateless requests that have to authenticate, decode, load, and encode data on every request. The flipside is that LiveView uses more memory on the server compared to stateless requests. Use cases There are many use cases where LiveView is an excellent fit right now: Handling of user interaction and inputs, buttons, and forms - such as input validation, dynamic forms, autocomplete, etc; Events and updates pushed by server - such as notifications, dashboards, etc; There are other cases that have limited support but will become first-class as we further develop LiveView: Page and data navigation - such as navigating between pages, pagination, etc can be built with LiveView but currently you will lose the back/forward button, and the ability to link to pages as you navigate. Support for pushState is on the roadmap; Cumulative and always growing data - chat applications, logs, and similar can be built with LiveView but currently you have to keep in the server a copy of all messages shown in the client as there is no append/prepend operation. Support for append/prepend is on the roadmap; Transitions and loading states - the LiveView programming model provides a good foundation for transitions and loading states since any UI change done after a user action is undone once the server sends the update for said action. For example, it is relatively straight-forward to click a button that changes itself in a way that is automatically undone when the update arrives. This is especially important as user feedback when latency is involved. A complete feature set for modelling those states is coming in future versions; There are also use cases which are a bad fit for LiveView: Animations - animations, menus, and general events that do not need the server in the first place are a bad fit for LiveView, as they can be achieved purely with CSS and/or CSS transitions; Optimistic UIs - once we add transitions and loading states, many of the building blocks necessary for building optimistic UIs will be part of LiveView, but since optimistic UIs are about doing work on the client while the server is unavailable, complete support for Optimistic UIs cannot be achieved without also writing JavaScript for the cases the server is not available; Life-cycle A LiveView begins as a regular HTTP request and HTML response, and then upgrades to a stateful view on client connect, guaranteeing a regular HTML page even if JavaScript is disabled. Any time a stateful view changes or updates its socket assigns, it is automatically re-rendered and the updates are pushed to the client. You begin by rendering a LiveView from your router or controller while providing session data to the view, which represents request info necessary for the view, such as params, cookie session info, etc. The session is signed and stored on the client, then provided back to the server when the client connects, or reconnects to the stateful view. When a view is rendered from the controller, the mount/2 callback is invoked with the provided session data and the LiveView socket. The mount/2 callback wires up socket assigns necessary for rendering the view. After mounting, render/1 is invoked and the HTML is sent as a regular HTML response to the client. After rendering the static page with a signed session, LiveView connects from the client where stateful views are spawned to push rendered updates to the browser, and receive client events via phx bindings. Just like the controller flow, mount/2 is invoked with the signed session, and socket state, where mount assigns values for rendering. However, in the connected client case, a LiveView process is spawned on the server, pushes the result of render/1 to the client and continues on for the duration of the connection. If at any point during the stateful life-cycle a crash is encountered, or the client connection drops, the client gracefully reconnects to the server, passing its signed session back to mount/2. Example First, a LiveView requires two callbacks: mount/2 and render/1: defmodule AppWeb.ThermostatLive do use Phoenix.LiveView def render(assigns) do ~L&quot;&quot;&quot; Current temperature: &lt;%= @temperature %&gt; &quot;&quot;&quot; end def mount(%{id: id, current_user_id: user_id}, socket) do case Thermostat.get_user_reading(user_id, id) do {:ok, temperature} -&gt; {:ok, assign(socket, :temperature, temperature)} {:error, reason} -&gt; {:error, reason} end end end The render/1 callback receives the socket.assigns and is responsible for returning rendered content. You can use Phoenix.LiveView.sigil_L/2 to inline LiveView templates. If you want to use Phoenix.HTML helpers, remember to use Phoenix.HTML at the top of your LiveView. A separate .leex HTML template can also be rendered within your render/1 callback by delegating to an existing Phoenix.View module in your application. For example: defmodule AppWeb.ThermostatLive do use Phoenix.LiveView def render(assigns) do Phoenix.View.render(AppWeb.PageView, &quot;page.html&quot;, assigns) end end With a LiveView defined, you first define the socket path in your endpoint, and point it to Phoenix.LiveView.Socket: defmodule AppWeb.Endpoint do use Phoenix.Endpoint socket &quot;/live&quot;, Phoenix.LiveView.Socket ... end And configure its signing salt in the endpoint: config :my_app, AppWeb.Endpoint, ..., live_view: [signing_salt: ...] You can generate a secure, random signing salt with the mix phx.gen.secret 32 task. Next, decide where you want to use your LiveView. You can serve the LiveView directly from your router, passing along :session values if needed (see Phoenix.LiveView.Router for more options): defmodule AppWeb.Router do use Phoenix.Router import Phoenix.LiveView.Router scope &quot;/&quot;, AppWeb do live &quot;/thermostat&quot;, ThermostatLive, session: [:user_id] end end You can also live_render from any template: &lt;h1&gt;Temperature Control&lt;/h1&gt; &lt;%= Phoenix.LiveView.live_render(@conn, AppWeb.ThermostatLive, session: %{user_id: @user.id}) %&gt; Or you can live_render your view from any controller: defmodule AppWeb.ThermostatController do ... import Phoenix.LiveView.Controller def show(conn, %{&quot;id&quot; =&gt; id}) do live_render(conn, AppWeb.ThermostatLive, session: %{ id: id, current_user_id: get_session(conn, :user_id), }) end end As we saw in the life-cycle section, you pass :session data about the request to the view, such as the current user&#39;s id in the cookie session, and parameters from the request. A regular HTML response is sent with a signed token embedded in the DOM containing your LiveView session data. Next, your client code connects to the server: import LiveSocket from &quot;phoenix_live_view&quot; let liveSocket = new LiveSocket(&quot;/live&quot;) liveSocket.connect() Note: Comprehensive JavaScript client usage is covered in detail below. After the client connects, mount/2 will be invoked inside a spawned LiveView process. At this point, you can use connected?/1 to conditionally perform stateful work, such as subscribing to pubsub topics, sending messages, etc. For example, you can periodically update a LiveView with a timer: defmodule DemoWeb.ThermostatLive do use Phoenix.LiveView ... def mount(%{id: id, current_user_id: user_id}, socket) do if connected?(socket), do: :timer.send_interval(30000, self(), :update) case Thermostat.get_user_reading(user_id, id) do {:ok, temperature} -&gt; {:ok, assign(socket, temperature: temperature, id: id)} {:error, reason} -&gt; {:error, reason} end end def handle_info(:update, socket) do {:ok, temperature} = Thermostat.get_reading(socket.assigns.id) {:noreply, assign(socket, :temperature, temperature)} end end We used connected?(socket) on mount to send our view a message every 30s if the socket is in a connected state. We receive :update in a handle_info just like a GenServer, and update our socket assigns. Whenever a socket&#39;s assigns change, render/1 is automatically invoked, and the updates are sent to the client. LiveEEx Templates Phoenix.LiveView&#39;s built-in templates provided by the .leex extension or ~L sigil, stands for Live EEx. They are similar to regular .eex templates except they are designed to minimize the amount of data sent over the wire by splitting static from dynamic parts and also tracking changes. When you first render a .leex template, it will send all of the static and dynamic parts of the template to the client. After that, any change you do on the server will now send only the dynamic parts, and only if those parts have changed. The tracking of changes is done via assigns. Imagine this template: &lt;div id=&quot;user_&lt;%= @user.id %&gt;&quot;&gt; &lt;%= @user.name %&gt; &lt;/div&gt; If the @user assign changes, then LiveView will re-render only the @user.id and @user.name and send them to the browser. The change tracking also works when rendering other templates, as long as they are also .leex templates and as long as all assigns are passed to the child/inner template: &lt;%= render &quot;child_template.html&quot;, assigns %&gt; The assign tracking feature also implies that you MUST avoid performing direct operations in the template. For example, if you perform a database query in your template: &lt;%= for user &lt;- Repo.all(User) do %&gt; &lt;%= user.name %&gt; &lt;% end %&gt; Then Phoenix will never re-render the section above, even if the amount of users in the database changes. Instead, you need to store the users as assigns in your LiveView before it renders the template: assign(socket, :users, Repo.all(User)) Generally speaking, data loading should never happen inside the template, regardless if you are using LiveView or not. The difference is that LiveView enforces those as best practices. Temporary assigns By default, all LiveView assigns are stateful, which enables change tracking and stateful interactions. In some cases, it&#39;s useful to mark assigns as temporary, meaning they will be set to nil after each update, allowing otherwise large, but infrequently updated values to be discarded after the client has been patched. Note: this requires refetching/recomputing the temporary assigns should they need accessed in future callbacks. To mark assigns as temporary, use configure_temporary_assigns/2: def mount(_session, socket) do description = fetch_large_description() {:ok, socket |&gt; assign(description: description) |&gt; configure_temporary_assigns([:description])} end Bindings Phoenix supports DOM element bindings for client-server interaction. For example, to react to a click on a button, you would render the element: &lt;button phx-click=&quot;inc_temperature&quot;&gt;+&lt;/button&gt; Then on the server, all LiveView bindings are handled with the handle_event callback, for example: def handle_event(&quot;inc_temperature&quot;, _value, socket) do {:ok, new_temp} = Thermostat.inc_temperature(socket.assigns.id) {:noreply, assign(socket, :temperature, new_temp)} end Click Events The phx-click binding is used to send click events to the server. When any client event, such as a phx-click click is pushed, the value sent to the server will be chosen with the following priority: An optional &quot;phx-value&quot; binding on the clicked element which is sent as is and handled as a string on the server Any number of optional phx-value- prefixed attributes, such as: &lt;div phx-click=&quot;inc&quot; phx-value-myvar1=&quot;val1&quot; phx-value-mvar2=&quot;val2&quot;&gt; will send the following map of params to the server: def handle_event(&quot;inc&quot;, %{&quot;myvar1&quot; =&gt; &quot;val1&quot;, &quot;myvar2&quot; =&gt; &quot;val2&quot;}, socket) do If the phx-value- prefixed is used, the server payload will also contain a &quot;value&quot; if the element&#39;s value attribute exists. When receiving a map on the server, the payload will also contain metadata of the client event, containing all literal keys of the event object, such as a click event&#39;s clientX, a keydown event&#39;s keyCode, etc. Focus and Blur Events Focus and blur events may be bound to DOM elements that emit such events, using the phx-blur, and phx-focus bindings, for example: &lt;input name=&quot;email&quot; phx-focus=&quot;myfocus&quot; phx-blur=&quot;myblur&quot;/&gt; To detect when the page itself has received focus or blur, phx-target may be specified as &quot;window&quot;. Like other bindings, a phx-value can be provided on the bound element, otherwise the input&#39;s value will be used. For example: &lt;div class=&quot;container&quot; phx-focus=&quot;page-active&quot; phx-blur=&quot;page-inactive&quot; phx-target=&quot;window&quot;&gt; ... &lt;/div&gt; Form Events To handle form changes and submissions, use the phx-change and phx-submit events. In general, it is preferred to handle input changes at the form level, where all form fields are passed to the LiveView&#39;s callback given any single input change. For example, to handle real-time form validation and saving, your template would use both phx_change and phx_submit bindings: &lt;%= f = form_for @changeset, &quot;#&quot;, [phx_change: :validate, phx_submit: :save] %&gt; &lt;%= label f, :username %&gt; &lt;%= text_input f, :username %&gt; &lt;%= error_tag f, :username %&gt; &lt;%= label f, :email %&gt; &lt;%= text_input f, :email %&gt; &lt;%= error_tag f, :email %&gt; &lt;%= submit &quot;Save&quot; %&gt; &lt;/form&gt; Next, your LiveView picks up the events in handle_event callbacks: def render(assigns) ... def mount(_session, socket) do {:ok, assign(socket, %{changeset: Accounts.change_user(%User{})})} end def handle_event(&quot;validate&quot;, %{&quot;user&quot; =&gt; params}, socket) do changeset = %User{} |&gt; Accounts.change_user(params) |&gt; Map.put(:action, :insert) {:noreply, assign(socket, changeset: changeset)} end def handle_event(&quot;save&quot;, %{&quot;user&quot; =&gt; user_params}, socket) do case Accounts.create_user(user_params) do {:ok, user} -&gt; {:stop, socket |&gt; put_flash(:info, &quot;user created&quot;) |&gt; redirect(to: Routes.user_path(AppWeb.Endpoint, AppWeb.User.ShowView, user))} {:error, %Ecto.Changeset{} = changeset} -&gt; {:noreply, assign(socket, changeset: changeset)} end end The validate callback simply updates the changeset based on all form input values, then assigns the new changeset to the socket. If the changeset changes, such as generating new errors, render/1 is invoked and the form is re-rendered. Likewise for phx-submit bindings, the same callback is invoked and persistence is attempted. On success, a :stop tuple is returned and the socket is annotated for redirect with Phoenix.LiveView.redirect/2, otherwise the socket assigns are updated with the errored changeset to be re-rerendered for the client. Note: For proper form error tag updates, the error tag must specify which input it belongs to. This is accomplished with the data-phx-error-for attribute. For example, your AppWeb.ErrorHelpers may use this function: def error_tag(form, field) do Enum.map(Keyword.get_values(form.errors, field), fn error -&gt; content_tag(:span, translate_error(error), class: &quot;help-block&quot;, data: [phx_error_for: input_id(form, field)] ) end) end Key Events The onkeydown, and onkeyup events are supported via the phx-keydown, and phx-keyup bindings. When pushed, the value sent to the server will contain all the client event object&#39;s literal metadata, such as keyCode, which, etc. By default, the bound element will be the event listener, but an optional phx-target may be provided which may be &quot;document&quot;, &quot;window&quot;, or the DOM id of a target element, for example: @up_key 38 @down_key 40 def render(assigns) do ~L&quot;&quot;&quot; &lt;div id=&quot;thermostat&quot; phx-keyup=&quot;update_temp&quot; phx-target=&quot;document&quot;&gt; Current temperature: &lt;%= @temperature %&gt; &lt;/div&gt; &quot;&quot;&quot; end def handle_event(&quot;update_temp&quot;, @up_key, socket) do {:ok, new_temp} = Thermostat.inc_temperature(socket.assigns.id) {:noreply, assign(socket, :temperature, new_temp)} end def handle_event(&quot;update_temp&quot;, @down_key, socket) do {:ok, new_temp} = Thermostat.dec_temperature(socket.assigns.id) {:noreply, assign(socket, :temperature, new_temp)} end def handle_event(&quot;update_temp&quot;, _key, socket) do {:noreply, socket} end Live navigation The live_link/2 and live_redirect/2 functions allow page navigation using the browser&#39;s pushState API. With live navigation, the page is updated without a full page reload. To use live navigation, simply replace your existing Phoenix.HTML.link/3 and Phoenix.LiveView.redirect/2 calls with their live counterparts. For example, in a template you may write: &lt;%= live_link &quot;next&quot;, to: Routes.live_path(@socket, MyLive, @page + 1) %&gt; or in a LiveView: {:noreply, live_redirect(socket, to: Routes.live_path(socket, MyLive, page + 1))} When a live link is clicked, the following control flow occurs: if the route belongs to the existing root LiveView and the LiveView is defined in your application&#39;s router, the handle_params/3 callback is invoked without mounting a new LiveView. See the next section. if the route belongs to a different LiveView than the currently running root, then the existing root LiveView is shutdown, and an Ajax request is made to request the necessary information about the new LiveView, without performing a full static render (which reduces latency and improves performance). Once information is retrieved, the new LiveView is mounted. handle_params/3 The handle_params/3 callback is invoked after mount/2. It receives the request path parameters and the query parameters as first argument, the url as second, and the socket as third. As any other handle_* callback, changes to the state inside handle_params/3 will trigger a server render. To avoid building a new LiveView whenever a live link is clicked or whenever a live redirect happens, LiveView also invokes handle_params/3 on an existing LiveView when performing live navigation as long as: you are navigating to the same root live view you are currently on said LiveView is defined in your router For example, imagine you have a UserTable LiveView to show all users in the system and you define it in the router as: live &quot;/users&quot;, UserTable Now to add live sorting, you could do: &lt;%= live_link &quot;Sort by name&quot;, to: Routes.live_path(@socket, UserTable, %{sort_by: &quot;name&quot;}) %&gt; When clicked, since we are navigating to the current LiveView, handle_params/3 will be invoked. Remember you should never trust received params, so we can use the callback to validate the user input and change the state accordingly: def handle_params(params, _uri, socket) do case params[&quot;sort_by&quot;] do sort_by when sort_by in ~w(name company) -&gt; {:noreply, socket |&gt; assign(:sort_by, sort) |&gt; recompute_users()} _ -&gt; {:noreply, socket} end end Replace page address LiveView also allows the current browser URL to be replaced. This is useful when you want certain events to change the URL but without polluting the browser&#39;s history. For example, imagine there is a form that changes some page state when submitted. If those changes are not persisted in a database or similar, as soon as the user refreshes the page, navigates away, or shares the URL with someone else, said changes will be lost. To address this, users can invoke live_redirect/2. The idea is, once the form data is received, we do not change the state, instead we perform a live redirect to ourselves with the new URL. Since we are navigating to ourselves, handle_params/3 will be called with the new parameters, which we can then use to compute state and re-render the page. For example, let&#39;s change the &quot;sort by&quot; example from the previous page to perform sorting through a form. In other words, instead of sorting by clicking a &quot;Sort by name&quot; button, we will have a form with 2 radio buttons, that allows you to choose between sorting by name or company. Once the form is submitted, we can compute the new URL: def handle_event(&quot;sorting&quot;, params, socket) do {:noreply, live_redirect(socket, to: Routes.live_path(socket, __MODULE__, params))} end Now with a handle_params/3 implementation similar to the one in the previous section, we will recompute the users based on the new params and perform a server render if there are any changes. Both live_link/2 and live_redirect/2 support the replace: true option. This option can be used when you want to change the current url without polluting the browser&#39;s history: def handle_event(&quot;sorting&quot;, params, socket) do {:noreply, live_redirect(socket, to: Routes.live_path(socket, __MODULE__, params), replace: true)} end JavaScript Client Specific As seen earlier, you start by instantiating a single LiveSocket instance to enable LiveView client/server interaction, for example: import LiveSocket from &quot;phoenix_live_view&quot; let liveSocket = new LiveSocket(&quot;/live&quot;) liveSocket.connect() All options are passed directly to the Phoenix.Socket constructor, except for the following LiveView specific options: bindingPrefix - the prefix to use for phoenix bindings. Defaults &quot;phx-&quot; params - the connect_params to pass to the view&#39;s mount callback. May be a literal object or closure returning an object. When a closure is provided, the function receives the view&#39;s phx-view name. hooks – a reference to a user-defined hooks namespace, containing client callbacks for server/client interop. See the interop section below for details. Forms and input handling The JavaScript client is always the source of truth for current input values. For any given input with focus, LiveView will never overwrite the input&#39;s current value, even if it deviates from the server&#39;s rendered updates. This works well for updates where major side effects are not expected, such as form validation errors, or additive UX around the user&#39;s input values as they fill out a form. For these use cases, the phx-change input does not concern itself with disabling input editing while an event to the server is inflight. When a phx-change event is sent to the server, a &quot;_target&quot; param will be in the root payload containing the keyspace of the input name which triggered the change event. For example, if the following input triggered a change event: &lt;input name=&quot;user[username]&quot;/&gt; The server&#39;s handle_event/3 would receive a payload: %{&quot;_target&quot; =&gt; [&quot;user&quot;, &quot;username&quot;], &quot;user&quot; =&gt; %{&quot;name&quot; =&gt; &quot;Name&quot;}} The phx-submit event is used for form submissions where major side-effects typically happen, such as rendering new containers, calling an external service, or redirecting to a new page. For these use-cases, the form inputs are set to readonly on submit, and any submit button is disabled until the client gets an acknowledgment that the server has processed the phx-submit event. Following an acknowledgment, any updates are patched to the DOM as normal, and the last input with focus is restored if the user has not otherwise focused on a new input during submission. To handle latent form submissions, any HTML tag can be annotated with phx-disable-with, which swaps the element&#39;s innerText with the provided value during form submission. For example, the following code would change the &quot;Save&quot; button to &quot;Saving...&quot;, and restore it to &quot;Save&quot; on acknowledgment: &lt;button type=&quot;submit&quot; phx-disable-with=&quot;Saving...&quot;&gt;Save&lt;/button&gt; Loading state and Errors By default, the following classes are applied to the live view&#39;s parent container: &quot;phx-connected&quot; - applied when the view has connected to the server &quot;phx-disconnected&quot; - applied when the view is not connected to the server &quot;phx-error&quot; - applied when an error occurs on the server. Note, this class will be applied in conjunction with &quot;phx-disconnected&quot; if connection to the server is lost. When a form bound with phx-submit is submitted, the phx-loading class is applied to the form, which is removed on update. Custom JS Interop and client controlled DOM A container can be marked with phx-update, allowing the DOM patch operations to avoid updating or removing portions of the LiveView, or to append or prepend the updates rather than replacing the existing contents. This is useful for client-side interop with existing libraries that do their own DOM operations. The following phx-update values are supported: replace - the default operation. Replaces the element with the contents ignore - ignores updates the DOM regardless of new content changes append - append the new DOM contents instead of replacing prepend - prepend the new DOM contents instead of replacing To handle custom client-side javascript when an element is added, updated, or removed by the server, a hook object may be provided with the following life-cycle callbacks: mounted - the element has been added to the DOM and its server LiveView has finished mounting updated - the element has been updated in the DOM by the server destroyed - the element has been removed from the page, either by a parent update, or the parent being removed entirely disconnected - the element&#39;s parent LiveView has disconnected from the server reconnected - the element&#39;s parent LiveView has reconnected to the server In addition to the callbacks, the callbacks contain the following attributes in scope: el - attribute referencing the bound DOM node, viewName - attribute matching the dom node&#39;s phx-view value pushEvent(event, payload) - method to push an event from the client to the LiveView server For example, a controlled input for phone-number formatting would annotate their markup: Then a hook callback object can be defined and passed to the socket: let Hooks = {} Hooks.PhoneNumber = { mounted(){ this.el.addEventListener(&quot;input&quot;, e =&gt; { let match = this.el.value.replace(/D/g, &quot;&quot;).match(/^(\d{3})(\d{3})(\d{4})$/) if(match) { this.el.value = `${match[1]}-${match[2]}-${match[3]}` } }) } } let liveSocket = new LiveSocket(&quot;/socket&quot;, {hooks: Hooks}) ..."},{"ref":"Phoenix.LiveView.html#assign/2","title":"Phoenix.LiveView.assign/2","type":"function","doc":""},{"ref":"Phoenix.LiveView.html#assign/3","title":"Phoenix.LiveView.assign/3","type":"function","doc":"Adds key value pairs to socket assigns. A single key value pair may be passed, or a keyword list of assigns may be provided to be merged into existing socket assigns. Examples iex&gt; assign(socket, :name, &quot;Elixir&quot;) iex&gt; assign(socket, name: &quot;Elixir&quot;, logo: &quot;💧&quot;)"},{"ref":"Phoenix.LiveView.html#assign_new/3","title":"Phoenix.LiveView.assign_new/3","type":"function","doc":"Assigns a value into the socket only if it does not exist. Useful for lazily assigning values and referencing parent assigns. Referencing parent assigns When a LiveView is mounted in a disconnected state, the Plug.Conn assigns will be available for reference via assign_new/3, allowing assigns to be shared for the initial HTTP request. On connected mount, the assign_new/3 would be invoked, and the LiveView would use its session to rebuild the originally shared assign. Likewise, nested LiveView children have access to their parent&#39;s assigns on mount using assign_new, which allows assigns to be shared down the nested LiveView tree. Examples # controller conn |&gt; assign(:current_user, user) |&gt; LiveView.Controller.live_render(MyLive, session: %{user_id: user.id}) # LiveView mount def mount(%{user_id: user_id}, socket) do {:ok, assign_new(socket, :current_user, fn -&gt; Accounts.get_user!(user_id) end)} end"},{"ref":"Phoenix.LiveView.html#configure_temporary_assigns/2","title":"Phoenix.LiveView.configure_temporary_assigns/2","type":"function","doc":"Configures the temporary assigns keys in the socket on mount. Temporary assigns are not kept after they are rendered. This saves server memory, but requires future access to refetch necessary data on-demand. Examples def mount(_session, socket) do description = fetch_large_description() {:ok, socket |&gt; assign(description: description) |&gt; configure_temporary_assigns([:description])} end"},{"ref":"Phoenix.LiveView.html#connected?/1","title":"Phoenix.LiveView.connected?/1","type":"function","doc":"Returns true if the socket is connected. Useful for checking the connectivity status when mounting the view. For example, on initial page render, the view is mounted statically, rendered, and the HTML is sent to the client. Once the client connects to the server, a LiveView is then spawned and mounted statefully within a process. Use connected?/1 to conditionally perform stateful work, such as subscribing to pubsub topics, sending messages, etc. Examples defmodule DemoWeb.ClockLive do use Phoenix.LiveView ... def mount(_session, socket) do if connected?(socket), do: :timer.send_interval(1000, self(), :tick) {:ok, assign(socket, date: :calendar.local_time())} end def handle_info(:tick, socket) do {:noreply, assign(socket, date: :calendar.local_time())} end end"},{"ref":"Phoenix.LiveView.html#get_connect_params/1","title":"Phoenix.LiveView.get_connect_params/1","type":"function","doc":"Accesses the connect params sent by the client for use on connected mount. Connect params are only sent when the client connects to the server and only remain available during mount. nil is returned when called in a disconnected state and a RuntimeError is raised if called after mount. Examples def mount(_session, socket) do {:ok, assign(socket, width: get_connect_params(socket)[&quot;width&quot;] || @width)} end"},{"ref":"Phoenix.LiveView.html#c:handle_call/3","title":"Phoenix.LiveView.handle_call/3","type":"callback","doc":""},{"ref":"Phoenix.LiveView.html#c:handle_event/3","title":"Phoenix.LiveView.handle_event/3","type":"callback","doc":""},{"ref":"Phoenix.LiveView.html#c:handle_info/2","title":"Phoenix.LiveView.handle_info/2","type":"callback","doc":""},{"ref":"Phoenix.LiveView.html#c:handle_params/3","title":"Phoenix.LiveView.handle_params/3","type":"callback","doc":""},{"ref":"Phoenix.LiveView.html#live_link/1","title":"Phoenix.LiveView.live_link/1","type":"function","doc":"Generates a live link for HTML5 pushState based navigation without page reloads. Options :to - the required path to link to. :replace - the flag to replace the current history or push a new state. Defaults false. All other options are forwarded to the anchor tag. Examples &lt;%= live_link &quot;next&quot;, to: Routes.live_path(@socket, MyLive, @page + 1) %&gt; &lt;%= live_link to: Routes.live_path(@socket, MyLive, dir: :asc), replace: false do %&gt; Sort By Price &lt;% end %&gt;"},{"ref":"Phoenix.LiveView.html#live_link/2","title":"Phoenix.LiveView.live_link/2","type":"function","doc":""},{"ref":"Phoenix.LiveView.html#live_redirect/2","title":"Phoenix.LiveView.live_redirect/2","type":"function","doc":"Annotates the socket for navigation without a page refresh. When navigating to a path which routes to your existing LiveView, the handle_params/3 callback is immediately invoked in your existing LiveView process to handle the change of URL state. For live redirects to external LiveViews, the existing LiveView is shutdown. Options :to - the required path to link to. :replace - the flag to replace the current history or push a new state. Defaults false. Examples {:noreply, live_redirect(socket, to: &quot;/&quot;)} {:noreply, live_redirect(socket, to: &quot;/&quot;, replace: true)}"},{"ref":"Phoenix.LiveView.html#live_render/3","title":"Phoenix.LiveView.live_render/3","type":"function","doc":"Renders a LiveView within an originating plug request or within a parent LiveView. Options :session - the map of session data to sign and send to the client. When connecting from the client, the LiveView will receive the signed session from the client and verify the contents before proceeding with mount/2. :container - the optional tuple for the HTML tag and DOM attributes to be used for the LiveView container. For example: {:li, style: &quot;color: blue;&quot;} :child_id - the ID to uniquely identify a child LiveView when live rendering children of the same type. Examples # within eex template &lt;%= live_render(@conn, MyApp.ThermostatLive) %&gt; # within leex template &lt;%= live_render(@socket, MyApp.ThermostatLive) %&gt;"},{"ref":"Phoenix.LiveView.html#c:mount/2","title":"Phoenix.LiveView.mount/2","type":"callback","doc":""},{"ref":"Phoenix.LiveView.html#put_flash/3","title":"Phoenix.LiveView.put_flash/3","type":"function","doc":"Adds a flash message to the socket to be displayed on redirect. Note: the Phoenix.LiveView.Flash plug must be plugged in your browser&#39;s pipeline for flash to be supported, for example: pipeline :browser do plug :accepts, [&quot;html&quot;] plug :fetch_session plug Phoenix.LiveView.Flash ... end Examples iex&gt; put_flash(socket, :info, &quot;It worked!&quot;) iex&gt; put_flash(socket, :error, &quot;You can&#39;t access that page&quot;)"},{"ref":"Phoenix.LiveView.html#redirect/2","title":"Phoenix.LiveView.redirect/2","type":"function","doc":"Annotates the socket for redirect to a destination path. Note: LiveView redirects rely on instructing client to perform a window.location update on the provided redirect location. TODO support :external and validation :to is a local path Options :to - the path to redirect to"},{"ref":"Phoenix.LiveView.html#c:render/1","title":"Phoenix.LiveView.render/1","type":"callback","doc":""},{"ref":"Phoenix.LiveView.html#sigil_L/2","title":"Phoenix.LiveView.sigil_L/2","type":"macro","doc":"Provides ~L sigil with HTML safe Live EEx syntax inside source files. iex&gt; ~L&quot;&quot;&quot; ...&gt; Hello &lt;%= &quot;world&quot; %&gt; ...&gt; &quot;&quot;&quot; {:safe, [&quot;Hello &quot;, &quot;world&quot;, &quot;\\n&quot;]}"},{"ref":"Phoenix.LiveView.html#c:terminate/2","title":"Phoenix.LiveView.terminate/2","type":"callback","doc":""},{"ref":"Phoenix.LiveView.html#update/3","title":"Phoenix.LiveView.update/3","type":"function","doc":"Updates an existing key in the socket assigns. The update function receives the current key&#39;s value and returns the updated value. Raises if the key does not exist. Examples iex&gt; update(socket, :count, fn count -&gt; count + 1 end) iex&gt; update(socket, :count, &amp;(&amp;1 + 1))"},{"ref":"Phoenix.LiveView.html#t:from/0","title":"Phoenix.LiveView.from/0","type":"type","doc":""},{"ref":"Phoenix.LiveView.html#t:unsigned_params/0","title":"Phoenix.LiveView.unsigned_params/0","type":"type","doc":""},{"ref":"Phoenix.LiveView.Comprehension.html","title":"Phoenix.LiveView.Comprehension","type":"module","doc":"The struct returned by for-comprehensions in .leex templates. See a description about its fields and use cases in Phoenix.LiveView.Engine docs."},{"ref":"Phoenix.LiveView.Comprehension.html#t:t/0","title":"Phoenix.LiveView.Comprehension.t/0","type":"type","doc":""},{"ref":"Phoenix.LiveView.Controller.html","title":"Phoenix.LiveView.Controller","type":"module","doc":"The Controller for LiveView rendering."},{"ref":"Phoenix.LiveView.Controller.html#live_render/3","title":"Phoenix.LiveView.Controller.live_render/3","type":"function","doc":"Renders a live view from a Plug request and sends an HTML response. Options :session - the map of session data to sign and send to the client. When connecting from the client, the live view will receive the signed session from the client and verify the contents before proceeding with mount/2. Before render the @live_view_module assign will be added to the connection assigns for reference. Examples defmodule ThermostatController do ... import Phoenix.LiveView.Controller def show(conn, %{&quot;id&quot; =&gt; thermostat_id}) do live_render(conn, ThermostatLive, session: %{ thermostat_id: id, current_user_id: get_session(conn, :user_id), }) end end"},{"ref":"Phoenix.LiveView.Engine.html","title":"Phoenix.LiveView.Engine","type":"module","doc":"The .leex (Live EEx) template engine that tracks changes. On the docs below, we will explain how it works internally. For user-facing documentation, see Phoenix.LiveView. Phoenix.LiveView.Rendered Whenever you render a .leex template, it returns a Phoenix.LiveView.Rendered structure. This structure has three fields: :static, :dynamic and :fingerprint. The :static field is a list of literal strings. This allows the Elixir compiler to optimize this list and avoid allocating its strings on every render. The :dynamic field contains a list of dynamic content. Each element in the list is either one of: iodata - which is the dynamic content nil - the dynamic content did not change, see &quot;Tracking changes&quot; below another Phoenix.LiveView.Rendered struct, see &quot;Nesting and fingerprinting&quot; below a Phoenix.LiveView.Comprehension struct, see &quot;Comprehensions&quot; below When you render a .leex template, you can convert the rendered structure to iodata by intercalating the static and dynamic fields, always starting with a static entry followed by a dynamic entry. The last entry will always be static too. So the following structure: %Phoenix.LiveView.Rendered{ static: [&quot;foo&quot;, &quot;bar&quot;, &quot;baz&quot;], dynamic: [&quot;left&quot;, &quot;right&quot;] } Results in the following content to be sent over the wire as iodata: [&quot;foo&quot;, &quot;left&quot;, &quot;bar&quot;, &quot;right&quot;, &quot;baz&quot;] This is also what calling Phoenix.HTML.Safe.to_iodata/1 with a Phoenix.LiveView.Rendered structure returns. Of course, the benefit of .leex templates is exactly that you do not need to send both static and dynamic segments every time. So let&#39;s talk about tracking changes. Tracking changes By default, a .leex template does not track changes. Change tracking can be enabled by passing a socket with two keys :fingerprints and :changed. If the :fingerprints matches the template fingerprint, then the :changed map is used. The map should contain the name of any changed field as key and the boolean true as value. If a field is not listed in :changed, then it is always considered unchanged. If a field is unchanged and .leex believes a dynamic expression no longer needs to be computed, its value in the dynamic list will be nil. This information can be leveraged to avoid sending data to the client. Nesting and fingerprinting Phoenix.LiveView also tracks changes across .leex templates. Therefore, if your view has this: &lt;%= render &quot;form.html&quot;, assigns %&gt; Phoenix will be able to track what is static and dynamic across templates, as well as what changed. A rendered nested .leex template will appear in the dynamic list as another Phoenix.LiveView.Rendered structure, which must be handled recursively. However, because the rendering of live templates can be dynamic in itself, it is important to distinguish which .leex template was rendered. For example, imagine this code: &lt;%= if something?, do: render(&quot;one.html&quot;, assigns), else: render(&quot;other.html&quot;, assigns) %&gt; To solve this, all Phoenix.LiveView.Rendered structs also contain a fingerprint field that uniquely identifies it. If the fingerprints are equal, you have the same template, and therefore it is possible to only transmit its changes. Comprehensions Another optimization done by .leex templates is to track comprehensions. If your code has this: &lt;%= for point &lt;- @points do %&gt; x: &lt;%= point.x %&gt; y: &lt;%= point.y %&gt; &lt;% end %&gt; Instead of rendering all points with both static and dynamic parts, it returns a Phoenix.LiveView.Comprehension struct with the static parts, that are shared across all points, and a list of dynamics to be interpolated inside the static parts. If @points is a list with %{x: 1, y: 2} and %{x: 3, y: 4}, the expression above would return: %Phoenix.LiveView.Comprehension{ static: [&quot;\\n x: &quot;, &quot;\\n y: &quot;, &quot;\\n&quot;], dynamics: [ [&quot;1&quot;, &quot;2&quot;], [&quot;3&quot;, &quot;4&quot;] ] } This allows .leex templates to drastically optimize the data sent by comprehensions, as the static parts are emitted once, regardless of the number of items. The list of dynamics is always a list of iodatas, as we only perform change tracking at the root and never inside case, cond, comprehensions, etc. Similarly, comprehensions do not have fingerprints because they are only optimized at the root, so conditional evaluation, as the one seen in rendering, is not possible. The only possible outcome for a dynamic field that returns a comprehension is nil."},{"ref":"Phoenix.LiveView.Flash.html","title":"Phoenix.LiveView.Flash","type":"module","doc":"Fetches Phoenix LiveView flash messages from cookie token. Examples plug Phoenix.LiveView.Flash By default, the signing salt for the token is pulled from your endpoint&#39;s LiveView config, for example: config :my_app, MyAppWeb.Endpoint, ..., live_view: [signing_salt: ...] The :signing_salt option may also be passed directly to the plug."},{"ref":"Phoenix.LiveView.Flash.html#call/2","title":"Phoenix.LiveView.Flash.call/2","type":"function","doc":"Fetches the live flash from a token passed via client cookies."},{"ref":"Phoenix.LiveView.Rendered.html","title":"Phoenix.LiveView.Rendered","type":"module","doc":"The struct returned by .leex templates. See a description about its fields and use cases in Phoenix.LiveView.Engine docs."},{"ref":"Phoenix.LiveView.Rendered.html#t:t/0","title":"Phoenix.LiveView.Rendered.t/0","type":"type","doc":""},{"ref":"Phoenix.LiveView.Router.html","title":"Phoenix.LiveView.Router","type":"module","doc":"Provides LiveView routing for Phoenix routers."},{"ref":"Phoenix.LiveView.Router.html#live/3","title":"Phoenix.LiveView.Router.live/3","type":"macro","doc":"Defines a LiveView route. Layout When a layout isn&#39;t explicitly set, a default layout is inferred similar to controller actions. For example, the layout for the router MyAppWeb.Router would be inferred as MyAppWeb.LayoutView and would use the :app template. Options :session - the optional list of keys to pull out of the Plug connection session and into the LiveView session. The :path_params keys may also be provided to copy the plug path params into the session. Defaults to [:path_params]. For example, the following would copy the path params and Plug session current user ID into the LiveView session: [:path_params, :user_id, :remember_me] :layout - the optional tuple for specifying a layout to render the LiveView. Defaults to {LayoutView, :app} where LayoutView is relative to your application&#39;s namespace. :container - the optional tuple for the HTML tag and DOM attributes to be used for the LiveView container. For example: {:li, style: &quot;color: blue;&quot;} :as - optionally configures the named helper. Defaults to :live. Examples defmodule MyApp.Router use Phoenix.Router import Phoenix.LiveView.Router scope &quot;/&quot;, MyApp do pipe_through [:browser] live &quot;/thermostat&quot;, ThermostatLive live &quot;/clock&quot;, ClockLive, session: [:path_params, :user_id] live &quot;/dashboard&quot;, DashboardLive, layout: {MyApp.AlternativeView, &quot;app.html&quot;} end end iex&gt; MyApp.Router.Helpers.live_path(MyApp.Endpoint, MyApp.ThermostatLive) &quot;/thermostat&quot;"},{"ref":"Phoenix.LiveView.Socket.html","title":"Phoenix.LiveView.Socket","type":"module","doc":"The LiveView socket for Phoenix Endpoints."},{"ref":"Phoenix.LiveView.Socket.html#connect/3","title":"Phoenix.LiveView.Socket.connect/3","type":"function","doc":"Connects the Phoenix.Socket for a LiveView client."},{"ref":"Phoenix.LiveView.Socket.html#id/1","title":"Phoenix.LiveView.Socket.id/1","type":"function","doc":"Identifies the Phoenix.Socket for a LiveView client."},{"ref":"Phoenix.LiveViewTest.html","title":"Phoenix.LiveViewTest","type":"module","doc":"Conveniences for testing Phoenix live views. In LiveView tests, we interact with views via process communication in substitution of a browser. Like a browser, our test process receives messages about the rendered updates from the view which can be asserted against to test the life-cycle and behavior of live views and their children. LiveView Testing The life-cycle of a live view as outlined in the Phoenix.LiveView docs details how a view starts as a stateless HTML render in a disconnected socket state. Once the browser receives the HTML, it connects to the server and a new LiveView process is started, remounted in a connected socket state, and the view continues statefully. The LiveView test functions support testing both disconnected and connected mounts separately, for example: use Phoenix.ConnTest import Phoenix.LiveViewTest @endpoint MyEndpoint test &quot;disconnected and connected mount&quot;, %{conn: conn} do conn = get(conn, &quot;/my-path&quot;) assert html_response(conn, 200) =~ &quot;&lt;h1&gt;My Disconnected View&lt;/h1&gt;&quot; {:ok, view, html} = live(conn) end test &quot;redirected mount&quot;, %{conn: conn} do assert {:error, %{redirect: %{to: &quot;/somewhere&quot;}}} = live(conn, &quot;my-path&quot;) end Here, we start by using the familiar Phoenix.ConnTest function, get/2 to test the regular HTTP get request which invokes mount with a disconnect socket. Next, live/1 is called with our sent connection to mount the view in a connected state, which starts our stateful LiveView process. In general, it&#39;s often more convenient to test the mounting of a view in a single step, provided you don&#39;t need the result of the stateless HTTP render. This is done with a single call to live/2, which performs the get step for us: test &quot;connected mount&quot;, %{conn: conn} do {:ok, view, html} = live(conn, &quot;/my-path&quot;) assert html =~ &quot;&lt;h1&gt;My Connected View&lt;/h1&gt;&quot; end Testing Events The browser can send a variety of events to a live view via phx- bindings, which are sent to the handle_event/3 callback. To test events sent by the browser and assert on the rendered side-effect of the event, use the render_* functions: render_click/3 - sends a phx-click event and value and returns the rendered result of the handle_event/3 callback. render_submit/3 - sends a form phx-submit event and value and returns the rendered result of the handle_event/3 callback. render_change/3 - sends a form phx-change event and value and returns the rendered result of the handle_event/3 callback. render_keydown/3 - sends a form phx-keydown event and value and returns the rendered result of the handle_event/3 callback. render_keyup/3 - sends a form phx-keyup event and value and returns the rendered result of the handle_event/3 callback. For example: {:ok, view, _html} = live(conn, &quot;/thermo&quot;) assert render_click(view, :inc) =~ &quot;The temperature is: 31℉&quot; assert render_click(view, :set_temp, 35) =~ &quot;The temperature is: 35℉&quot; assert render_submit(view, :save, %{deg: 30}) =~ &quot;The temperature is: 30℉&quot; assert render_change(view, :validate, %{deg: -30}) =~ &quot;invalid temperature&quot; assert render_keydown(view, :key, :ArrowUp) =~ &quot;The temperature is: 31℉&quot; assert render_keydown(view, :key, :ArrowDown) =~ &quot;The temperature is: 30℉&quot; Testing regular messages Live views are GenServer&#39;s under the hood, and can send and receive messages just like any other server. To test the side effects of sending or receiving messages, simply message the view and use the render function to test the result: send(view.pid, {:set_temp: 50}) assert render(view) =~ &quot;The temperature is: 50℉&quot; Testing shutdowns and stopping views Like all processes, views can shutdown normally or abnormally, and this can be tested with assert_remove/3. For example: send(view.pid, :boom) assert_remove view, {:shutdown, %RuntimeError{}} stop(view) assert_remove view, {:shutdown, :stop} Nested views can be removed by a parent at any time based on conditional rendering. In these cases, the removal of the view is detected by the browser, or our test client, and the child is shutdown gracefully. This can be tested in the same way as above: assert render(parent) =~ &quot;some content in child&quot; [child] = children(parent) send(parent.pid, :msg_that_removes_child) assert_remove child, _ refute render(parent) =~ &quot;some content in child&quot;"},{"ref":"Phoenix.LiveViewTest.html#assert_redirect/3","title":"Phoenix.LiveViewTest.assert_redirect/3","type":"macro","doc":"Asserts a redirect was peformed after execution of the provided function. Examples assert_redirect view, &quot;/path&quot;, fn -&gt; assert render_click(view, :event_that_triggers_redirect) end"},{"ref":"Phoenix.LiveViewTest.html#assert_remove/3","title":"Phoenix.LiveViewTest.assert_remove/3","type":"macro","doc":"Asserts a view was removed by a parent or shutdown itself. Examples [child1, child2] = children(parent_view) send(parent_view.pid, :msg_that_removes_child) assert_remove child1, _ assert_remove child2, {:shutdown, :removed}"},{"ref":"Phoenix.LiveViewTest.html#children/1","title":"Phoenix.LiveViewTest.children/1","type":"function","doc":"Returns the current list of children of the parent live view. Children are return in the order they appear in the rendered HTML. Examples {:ok, view, _html} = live(conn, &quot;/thermo&quot;) assert [clock_view] = children(view) assert render_click(clock_view, :snooze) =~ &quot;snoozing&quot;"},{"ref":"Phoenix.LiveViewTest.html#live/2","title":"Phoenix.LiveViewTest.live/2","type":"macro","doc":"Spawns a connected LiveView process. Accepts either a previously rendered %Plug.Conn{} or an unsent %Plug.Conn{}. The latter case is a conveience to perform the get/2 and connected mount in a single step. Options :connect_params - the map of params available in connected mount Examples {:ok, view, html} = live(conn, &quot;/path&quot;) assert view.module = MyLive assert html =~ &quot;the count is 3&quot; assert {:error, %{redirect: %{to: &quot;/somewhere&quot;}}} = live(conn, &quot;/path&quot;) {:ok, view, html} = conn |&gt; get(&quot;/path&quot;) |&gt; live()"},{"ref":"Phoenix.LiveViewTest.html#live/3","title":"Phoenix.LiveViewTest.live/3","type":"macro","doc":""},{"ref":"Phoenix.LiveViewTest.html#render/1","title":"Phoenix.LiveViewTest.render/1","type":"function","doc":"Returns the string of HTML of the rendered view."},{"ref":"Phoenix.LiveViewTest.html#render_blur/3","title":"Phoenix.LiveViewTest.render_blur/3","type":"function","doc":"Sends a blur event to the view and returns the rendered result. Examples {:ok, view, html} = live(conn, &quot;/thermo&quot;) assert html =~ &quot;The temp is: 30℉&quot; assert render_blur(view, :inactive) =~ &quot;Tap to wake&quot;"},{"ref":"Phoenix.LiveViewTest.html#render_change/3","title":"Phoenix.LiveViewTest.render_change/3","type":"function","doc":"Sends a form change event to the view and returns the rendered result. Examples {:ok, view, html} = live(conn, &quot;/thermo&quot;) assert html =~ &quot;The temp is: 30℉&quot; assert render_change(view, :validate, %{deg: 123}) =~ &quot;123 exceeds limits&quot;"},{"ref":"Phoenix.LiveViewTest.html#render_click/3","title":"Phoenix.LiveViewTest.render_click/3","type":"function","doc":"Sends a click event to the view and returns the rendered result. Examples {:ok, view, html} = live(conn, &quot;/thermo&quot;) assert html =~ &quot;The temperature is: 30℉&quot; assert render_click(view, :inc) =~ &quot;The temperature is: 31℉&quot;"},{"ref":"Phoenix.LiveViewTest.html#render_focus/3","title":"Phoenix.LiveViewTest.render_focus/3","type":"function","doc":"Sends a focus event to the view and returns the rendered result. Examples {:ok, view, html} = live(conn, &quot;/thermo&quot;) assert html =~ &quot;The temp is: 30℉&quot; assert render_blur(view, :inactive) =~ &quot;Tap to wake&quot; assert render_focus(view, :active) =~ &quot;Waking up...&quot;"},{"ref":"Phoenix.LiveViewTest.html#render_keydown/3","title":"Phoenix.LiveViewTest.render_keydown/3","type":"function","doc":"Sends a keydown event to the view and returns the rendered result. Examples {:ok, view, html} = live(conn, &quot;/thermo&quot;) assert html =~ &quot;The temp is: 30℉&quot; assert render_keyup(view, :inc, :ArrowUp) =~ &quot;The temp is: 32℉&quot;"},{"ref":"Phoenix.LiveViewTest.html#render_keyup/3","title":"Phoenix.LiveViewTest.render_keyup/3","type":"function","doc":"Sends a keyup event to the view and returns the rendered result. Examples {:ok, view, html} = live(conn, &quot;/thermo&quot;) assert html =~ &quot;The temp is: 30℉&quot; assert render_keyup(view, :inc, :ArrowUp) =~ &quot;The temp is: 32℉&quot;"},{"ref":"Phoenix.LiveViewTest.html#render_live_link/2","title":"Phoenix.LiveViewTest.render_live_link/2","type":"function","doc":"Simulates a live_link click to the view and returns the rendered result."},{"ref":"Phoenix.LiveViewTest.html#render_submit/3","title":"Phoenix.LiveViewTest.render_submit/3","type":"function","doc":"Sends a form submit event to the view and returns the rendered result. Examples {:ok, view, html} = live(conn, &quot;/thermo&quot;) assert html =~ &quot;The temp is: 30℉&quot; assert render_submit(view, :refresh, %{deg: 32}) =~ &quot;The temp is: 32℉&quot;"},{"ref":"Phoenix.LiveViewTest.html#stop/1","title":"Phoenix.LiveViewTest.stop/1","type":"function","doc":"Stops a LiveView process. Examples stop(view) assert_remove view, {:shutdown, :stop}"}]